package sun.swing;

import javax.swing.border.Border;
import javax.swing.Icon;
import java.awt.Color;
import java.awt.Insets;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import sun.awt.AppContext;

public class DefaultLookup
{
    private static final Object DEFAULT_LOOKUP_KEY;
    private static Thread currentDefaultThread;
    private static DefaultLookup currentDefaultLookup;
    private static boolean isLookupSet;
    
    public DefaultLookup() {
    }
    
    public static void setDefaultLookup(DefaultLookup defaultLookup) {
        synchronized (DefaultLookup.class) {
            if (!DefaultLookup.isLookupSet && defaultLookup == null) {
                return;
            }
            if (defaultLookup == null) {
                defaultLookup = new DefaultLookup();
            }
            DefaultLookup.isLookupSet = true;
            AppContext.getAppContext().put(DefaultLookup.DEFAULT_LOOKUP_KEY, defaultLookup);
            DefaultLookup.currentDefaultThread = Thread.currentThread();
            DefaultLookup.currentDefaultLookup = defaultLookup;
        }
    }
    
    public static Object get(JComponent component, ComponentUI componentUI, String key) {
        int isLookupSet;
        synchronized (DefaultLookup.class) {
            isLookupSet = DefaultLookup.isLookupSet;
        }
        if (isLookupSet == 0) {
            return UIManager.get(key, component.getLocale());
        }
        Thread currentThread = Thread.currentThread();
        Throwable currentDefaultLookup;
        synchronized (DefaultLookup.class) {
            if (currentThread == DefaultLookup.currentDefaultThread) {
                currentDefaultLookup = DefaultLookup.currentDefaultLookup;
            }
            else {
                currentDefaultLookup = (Object)AppContext.getAppContext().get(DefaultLookup.DEFAULT_LOOKUP_KEY);
                if (currentDefaultLookup == null) {
                    currentDefaultLookup = new DefaultLookup();
                    AppContext.getAppContext().put(DefaultLookup.DEFAULT_LOOKUP_KEY, currentDefaultLookup);
                }
                DefaultLookup.currentDefaultThread = currentThread;
                DefaultLookup.currentDefaultLookup = currentDefaultLookup;
            }
        }
        return currentDefaultLookup.getDefault(component, componentUI, key);
    }
    
    public static int getInt(JComponent component, ComponentUI componentUI, String s, int n) {
        Object value = get(component, componentUI, s);
        if (value == null || !(value instanceof Number)) {
            return n;
        }
        return ((Number)value).intValue();
    }
    
    public static int getInt(JComponent component, ComponentUI componentUI, String s) {
        return getInt(component, componentUI, s, -1);
    }
    
    public static Insets getInsets(JComponent component, ComponentUI componentUI, String s, Insets insets) {
        Object value = get(component, componentUI, s);
        if (value == null || !(value instanceof Insets)) {
            return insets;
        }
        return (Insets)value;
    }
    
    public static Insets getInsets(JComponent component, ComponentUI componentUI, String s) {
        return getInsets(component, componentUI, s, null);
    }
    
    public static boolean getBoolean(JComponent component, ComponentUI componentUI, String s, boolean b) {
        Object value = get(component, componentUI, s);
        if (value == null || !(value instanceof Boolean)) {
            return b;
        }
        return (boolean)value;
    }
    
    public static boolean getBoolean(JComponent component, ComponentUI componentUI, String s) {
        return getBoolean(component, componentUI, s, false);
    }
    
    public static Color getColor(JComponent component, ComponentUI componentUI, String s, Color color) {
        Object value = get(component, componentUI, s);
        if (value == null || !(value instanceof Color)) {
            return color;
        }
        return (Color)value;
    }
    
    public static Color getColor(JComponent component, ComponentUI componentUI, String s) {
        return getColor(component, componentUI, s, null);
    }
    
    public static Icon getIcon(JComponent component, ComponentUI componentUI, String s, Icon icon) {
        Object value = get(component, componentUI, s);
        if (value == null || !(value instanceof Icon)) {
            return icon;
        }
        return (Icon)value;
    }
    
    public static Icon getIcon(JComponent component, ComponentUI componentUI, String s) {
        return getIcon(component, componentUI, s, null);
    }
    
    public static Border getBorder(JComponent component, ComponentUI componentUI, String s, Border border) {
        Object value = get(component, componentUI, s);
        if (value == null || !(value instanceof Border)) {
            return border;
        }
        return (Border)value;
    }
    
    public static Border getBorder(JComponent component, ComponentUI componentUI, String s) {
        return getBorder(component, componentUI, s, null);
    }
    
    public Object getDefault(JComponent component, ComponentUI componentUI, String key) {
        return UIManager.get(key, component.getLocale());
    }
    
    static {
        DEFAULT_LOOKUP_KEY = new StringBuffer("DefaultLookup");
    }
}
